from flask import Blueprint
from ckan.common import config
import ckan.lib.datapreview as datapreview
import ckan.plugins.toolkit as toolkit
import ckan.plugins as p

from dcor_shared import DC_MIME_TYPES

from .jobs import create_preview_job
from .meta import render_metadata_html
from .route_funcs import dcpreview


class DCViewPlugin(p.SingletonPlugin):
    '''DC data view and route for *_preview.png'''
    p.implements(p.IBlueprint)
    p.implements(p.IConfigurer, inherit=True)
    p.implements(p.IResourceController, inherit=True)
    p.implements(p.IResourceView, inherit=True)

    # IBlueprint
    def get_blueprint(self):
        '''Return a Flask Blueprint object to be registered by the app.'''

        # Create Blueprint for plugin
        blueprint = Blueprint(self.name, self.__module__)

        # Add plugin url rules to Blueprint object
        rules = [
            ('/dataset/<uuid:id>/resource/<uuid:resource_id>/preview.jpg',
             'dcpreview',
             dcpreview),
        ]
        for rule in rules:
            blueprint.add_url_rule(*rule)
        return blueprint

    # IConfigurer
    def update_config(self, config):
        p.toolkit.add_template_directory(config, 'templates')
        p.toolkit.add_resource('assets', 'dc_view')

    # IResourceController
    def after_create(self, context, resource):
        """Generate preview image and html file"""
        jid = "-".join([resource["id"], resource["name"], "preview"])
        condense_jid = "-".join([resource["id"], resource["name"], "condense"])
        exts = config.get("ckan.plugins")
        if "dc_serve" in exts:
            # The dc_serve extension is available, which produces a condensed
            # dataset. We can use that for plotting.
            rq_kwargs = {"timeout": 60,
                         "job_id": jid,
                         "depends_on": condense_jid}
            queue = "dcor-normal"
        else:
            # The dc_serve extension is NOT available. Creating the preview
            # image might take longer.
            rq_kwargs = {"timeout": 1800,
                         "job_id": jid}
            queue = "dcor-long"
        toolkit.enqueue_job(create_preview_job,
                            [resource],
                            title="Create resource preview image",
                            queue=queue,
                            rq_kwargs=rq_kwargs)

    # IResourceView
    def info(self):
        return {'name': 'dc_view',
                'title': p.toolkit._('DC Info'),
                'icon': 'microscope',
                'iframed': False,
                'always_available': True,
                'default_title': p.toolkit._('DC Info'),
                }

    def can_view(self, data_dict):
        resource = data_dict['resource']
        mtype = resource.get('mimetype', '')
        same_domain = datapreview.on_same_domain(data_dict)
        if mtype in DC_MIME_TYPES and same_domain:
            return True
        else:
            return False

    def setup_template_variables(self, context, data_dict):
        preview_url = '/dataset/{}/resource/{}/preview.jpg'.format(
            data_dict['package']['id'], data_dict['resource']['id'])
        metadata_html = render_metadata_html(data_dict["resource"])
        return {
            'metadata_html': metadata_html,
            'preview_url': preview_url,
        }

    def view_template(self, context, data_dict):
        return 'dc_view.html'



import pathlib

import flask
from ckan.common import c
import ckan.lib.uploader as uploader
from ckan import logic
import ckan.model as model
import ckan.plugins.toolkit as toolkit


def dcpreview(id, resource_id):
    """Serve a preview image on disk

    `id` and `resource_id` are strings or uuids.
    """
    # Code borrowed from ckan/controllers/package.py:resource_download
    context = {'model': model, 'session': model.Session,
               'user': c.user, 'auth_user_obj': c.userobj}
    id = str(id)
    resource_id = str(resource_id)
    try:
        rsc = toolkit.get_action('resource_show')(context, {'id': resource_id})
        toolkit.get_action('package_show')(context, {'id': id})
    except (logic.NotFound, logic.NotAuthorized):
        toolkit.abort(404, toolkit._('Resource not found'))

    if rsc.get('url_type') == 'upload':
        upload = uploader.get_resource_uploader(rsc)
        filepath = pathlib.Path(upload.get_path(rsc['id']))
        jpg_file = filepath.with_name(filepath.name + "_preview.jpg")
        if not jpg_file.exists():
            toolkit.abort(404, toolkit._('Preview not found'))
        return flask.send_from_directory(jpg_file.parent, jpg_file.name)
    elif 'url' not in rsc:
        toolkit.abort(404, toolkit._('No download is available'))
    toolkit.redirect_to(rsc['url'])



from os.path import dirname, realpath, exists
from setuptools import setup, find_packages
from codecs import open  # To use a consistent encoding
import sys

author = "Paul Müller"
authors = [author]
description = 'Preview DC data in DCOR '
name = 'ckanext-dc_view'
year = "2020"

sys.path.insert(0, realpath(dirname(__file__))+"/" + "/".join(name.split("-")))
from _version import version


setup(
    name=name,
    version=version,
    description=description,
    long_description=open('README.rst').read() if exists('README.rst') else '',
    url='https://github.com/DCOR-dev/ckanext-dc_view',
    author=author,
    author_email='dev@craban.de',
    license='AGPLv3+',
    keywords=["CKAN", "DCOR", "RT-DC"],
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),
    package_dir={name: name},
    namespace_packages=['ckanext'],
    install_requires=[
        # CKAN extensions should not list dependencies here, but in a separate
        # ``requirements.txt`` file.
        #
        # http://docs.ckan.org/en/latest/extensions/best-practices.html#add-third-party-libraries-to-requirements-txt
    ],
    include_package_data=True,
    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    entry_points='''
        [ckan.plugins]
        dc_view=ckanext.dc_view.plugin:DCViewPlugin
    ''',
    # If you are changing from the default layout of your extension, you may
    # have to change the message extractors, you can read more about babel
    # message extraction at
    # http://babel.pocoo.org/docs/messages/#extraction-method-mapping-and-configuration
    message_extractors={
        'ckanext': [
            ('**.py', 'python', None),
            ('**.js', 'javascript', None),
            ('**/templates/**.html', 'ckan', None),
        ],
    },
    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'License :: OSI Approved :: GNU Affero General Public License v3 or ' \
        + 'later (AGPLv3+)',
        'Programming Language :: Python :: 3',
    ],
)
